@@ -1,10 +1,10 @@
 #!/usr/bin/env python
 from __future__ import division
 
-__version__ = "$Revision: 1.28 $"
+__version__ = "$Revision: 1.30 $"
 
+from distutils.spawn import find_executable
 from functools import partial
-import new
 import optparse
 import signal
 from subprocess import Popen, PIPE
@@ -101,8 +101,8 @@ class OptionBuilder(optparse.OptionParser):
         # list and the empty ones have to be eaten somehow
 
         res = []
-        for option_item in options.iteritems():
-            res.extend(self._build_option(*option_item))
+        for key in sorted(options):
+            res.extend(self._build_option(key, options[key]))
         return res
 
     @staticmethod
@@ -117,11 +117,17 @@ class OptionBuilder(optparse.OptionParser):
     def build_args(self, args=(), options={}):
         return self._build_options(options) + list(args)
 
-    def build_cmdline(self, args=(), options={}, prog=None):
+    def get_prog(self, prog):
+        """
+        virtual function to be overriden
+        """
         if prog is None:
             prog = self.prog
 
-        res = [prog]
+        return prog
+
+    def build_cmdline(self, args=(), options={}, prog=None):
+        res = [self.get_prog(prog)]
         res.extend(self.build_args(args, options))
 
         _write_log_exec(res)
@@ -255,11 +261,12 @@ class OptionBuilder_NoHyphenWithEquals(OptionBuilder):
 
         return ["%s=%s" % (option, value)]
 
+# XXX: this should go into another package
 class AddableMixinMetaclass(type):
     def __add__(cls, other):
         name = "(%s.%s + %s.%s)" % (cls.__module__, cls.__name__,
                                     other.__module__, other.__name__)
-        return new.classobj(name, (cls, other), {})
+        return type(name, (cls, other), {})
 
     __radd__ = __add__
 
@@ -281,13 +288,22 @@ class AddableMixin(object):
         if self.__class__.__name__.startswith("("):
             return "<%s object at 0x%x>" % (self.__class__.__name__, _id(self))
         else:
-            return object.__repr__(self)
+            return super(AddableMixin, self).__repr__(self)
 
-    def __new__(self, *args, **kwargs):
-        return object.__new__(self)
+    def __new__(cls, *args, **kwargs):
+        return super(AddableMixin, cls).__new__(cls, *args, **kwargs)
 
     def __init__(self, *args, **kwargs):
-        return super(object, self).__init__(*args, **kwargs)
+        # different depending on whether old- and new-style classes
+        # are being mixed, because object.__init__() does not call its
+        # superclass. I think this is a bug in object.
+        # XXX: verify and report bug upstream
+        if type(self).mro()[-1] is object:
+            supercls = super(AddableMixin, self)
+        else:
+            supercls = super(object, self)
+
+        return supercls.__init__(*args, **kwargs)
 
 class Mixin_ArgsFirst(AddableMixin):
     def build_args(self, args=(), options={}):
@@ -298,6 +314,17 @@ class Mixin_NoConvertUnderscore(AddableMixin):
     def convert_option_name(option):
         return option
 
+class Mixin_UseFullProgPath(AddableMixin):
+    def get_prog(self, prog):
+        prog = OptionBuilder.get_prog(self, prog)
+
+        res = find_executable(prog)
+
+        if res is None:
+            raise IOError("can't find %s in path" % prog)
+
+        return res
+
 def _setup_signals():
     res = {}
     for key, value in vars(signal).iteritems():
